package org.ihc.hwcir.sepsis.event_loop;

import java.util.Collection;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.definition.KnowledgePackage;
import org.drools.io.ResourceFactory;
import org.drools.logger.KnowledgeRuntimeLogger;
import org.drools.logger.KnowledgeRuntimeLoggerFactory;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.process.ProcessInstance;

/**
 * Sample Process Application
 * 
 */
public class ProcessApp {
    private StatefulKnowledgeSession ksession = null;

    public static void main(String[] args) {
        new ProcessApp();
    }

    // This code starts Hello.bpmn which waits for a signal, then DataDriveProcess sends that signal to complete it.
    private ProcessApp() {
        KnowledgeRuntimeLogger flogger = null;

        try {
            ksession = this.createKnowledgeSession();
            flogger = KnowledgeRuntimeLoggerFactory.newFileLogger(ksession, "auditlog");

            ProcessInstance pi = this.ksession.startProcess("Howdy");
            System.out.println("Process state: " + pi.getState());
            ksession.signalEvent("TestSignal", true, pi.getId());

            ksession.startProcess("DataDriveProcess");

            System.out.println("Process state: " + pi.getState());

        } finally {
            flogger.close();
            ksession.dispose();
        }

    }

    private StatefulKnowledgeSession createKnowledgeSession() {

        final KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();

        kbuilder.add(ResourceFactory.newClassPathResource("processes/Hello.bpmn", ProcessApp.class), ResourceType.BPMN2);
        kbuilder.add(ResourceFactory.newClassPathResource("processes/DataDriveProcess.bpmn", ProcessApp.class), ResourceType.BPMN2);

        if (kbuilder.hasErrors()) {
            System.out.println(kbuilder.getErrors().toString());
            throw new RuntimeException("Unable to compile.");
        }

        final Collection<KnowledgePackage> pkgs = kbuilder.getKnowledgePackages();

        final KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase();
        kbase.addKnowledgePackages(pkgs);

        final StatefulKnowledgeSession ksession = kbase.newStatefulKnowledgeSession();

        // Uncomment these to turn on Process/Rule logging.
        // ksession.addEventListener(new DebugAgendaEventListener());
        // ksession.addEventListener(new DebugWorkingMemoryEventListener());

        return ksession;
    }
}
